"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CsvStreamParser = exports.CsvCallbackParser = exports.CsvSyncParser = void 0;
const csv_parse_1 = require("csv-parse");
const sync_1 = require("csv-parse/sync");
const node_stream_1 = __importDefault(require("node:stream"));
const polyfill_1 = require("@js-temporal/polyfill");
var PlainDate = polyfill_1.Temporal.PlainDate;
const parseOptions = {
    columns: (headerRow) => {
        if (!Array.isArray(headerRow)) {
            throw new Error("Invalid CSV format");
        }
        const headerMapping = [
            ["Book Title", "bookTitle"],
            ["Chapter Title", "chapterTitle"],
            ["Date of Highlight", "dateOfHighlight"],
            ["Book URL", "bookURL"],
            ["Chapter URL", "chapterURL"],
            ["Annotation URL", "annotationURL"],
            ["Highlight", "highlight"],
            ["Color", "color"],
            ["Personal Note", "personalNote"],
        ];
        // Check if the record has the expected headers in the correct order
        if (!headerMapping.every(([expectedHeader], index) => headerRow[index] === expectedHeader)) {
            throw new Error("Invalid CSV format. Expected headers: " +
                headerMapping.map(([header]) => header).join(", "));
        }
        // Return the mapped column headers
        return headerMapping.map(([, header]) => header);
    },
    onRecord: (record) => {
        return Object.assign(Object.assign({}, record), { dateOfHighlight: PlainDate.from(record.dateOfHighlight) });
    },
};
/**
 * Parser to retrieve the highlights from a CSV file as an array.
 */
class CsvSyncParser {
    /**
     * Parses the input CSV file and returns the highlights as an array.
     */
    parse(input) {
        return (0, sync_1.parse)(input, parseOptions);
    }
}
exports.CsvSyncParser = CsvSyncParser;
/**
 * Parser to retrieve the highlights from a CSV file using a callback.
 */
class CsvCallbackParser {
    /**
     * Parses the input CSV file and returns the highlights as an array using a callback.
     */
    parse(input, callback) {
        (0, csv_parse_1.parse)(input, parseOptions, (err, records) => {
            if (err) {
                callback(err, []);
            }
            else {
                callback(null, records);
            }
        });
    }
}
exports.CsvCallbackParser = CsvCallbackParser;
/**
 * A stream transformer to parse CSV data into OreillyHighlight objects.
 */
class CsvStreamParser extends node_stream_1.default.Transform {
    constructor() {
        super({ objectMode: true });
        this.parser = (0, csv_parse_1.parse)(parseOptions);
        this.parser.on("readable", () => {
            let record;
            while ((record = this.parser.read()) !== null) {
                this.push(record);
            }
        });
        this.parser.on("error", (err) => this.emit("error", err));
    }
    _transform(chunk, encoding, callback) {
        this.parser.write(chunk, encoding, callback);
    }
    _flush(callback) {
        this.parser.end(callback);
    }
}
exports.CsvStreamParser = CsvStreamParser;
